"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("../types");
var DefinitionMetadata_1 = require("./DefinitionMetadata");
/**
 * Keep all [[DefinitionMetadata]] and [[GenericMetadata]].
 * Provide methods to query metadata with its associated class or graphql type name.
 */
var MetadataStorage = /** @class */ (function () {
    function MetadataStorage() {
        this.definitionMetadata = [];
        this.memoizedTypeInstanceMap = new Map();
        this.fieldReferences = [];
        this.inputFieldReferences = [];
    }
    /**
     * Add a new [[Metadata]] object to storage.
     * @param metadata A metadata object to register
     */
    MetadataStorage.prototype.register = function (metadata, definitionClass) {
        if (metadata instanceof DefinitionMetadata_1.DefinitionMetadata) {
            this.definitionMetadata.push({ definitionClass: definitionClass, metadata: metadata });
        }
        else {
            throw new Error("Cannot register to stroage: " + metadata);
        }
    };
    MetadataStorage.prototype.registerFieldReference = function (reference, definitionClass) {
        this.fieldReferences.push({ reference: reference, definitionClass: definitionClass });
    };
    MetadataStorage.prototype.registerInputFieldReference = function (reference, definitionClass) {
        this.inputFieldReferences.push({ reference: reference, definitionClass: definitionClass });
    };
    /**
     * Get a [[DefinitionMetadata]] object which is instance of the `metadataClass` and associated to `definitionClass`
     * @param metadataClass A [[DefinitionMetadata]] subclass to query
     * @param definitionClass A class associated with metadata to query
     */
    MetadataStorage.prototype.getDefinitionMetadata = function (metadataClass, definitionClass) {
        var entry = this.definitionMetadata.find(function (entry) { return entry.definitionClass === definitionClass; });
        if (!entry) {
            throw new Error("Cannot get " + metadataClass.name + " of " + definitionClass.name + " from MetadataStorage");
        }
        return entry;
    };
    /**
     * Get a [[DefinitionMetadata]] object by the name of the GraphQLType that the metadata will generate.
     * @param name The name of a GraphQLType which will be built from metadata to query
     */
    MetadataStorage.prototype.getDefinitionMetadataByName = function (name) {
        var entry = this.definitionMetadata.find(function (entry) { return entry.metadata.typeName === name; });
        if (!entry) {
            throw new Error("Cannot find metadata with given name: " + name);
        }
        return entry;
    };
    MetadataStorage.prototype.queryFieldReferences = function (definitionClass) {
        return this.fieldReferences.filter(function (entry) { return (definitionClass === entry.definitionClass || types_1.isSubClassOf(definitionClass, entry.definitionClass)); });
    };
    MetadataStorage.prototype.queryInputFieldReference = function (definitionClass) {
        return this.inputFieldReferences.filter(function (entry) { return (definitionClass === entry.definitionClass || types_1.isSubClassOf(definitionClass, entry.definitionClass)); });
    };
    return MetadataStorage;
}());
exports.MetadataStorage = MetadataStorage;
//# sourceMappingURL=MetadataStorage.js.map